/*
 * Copyright (C) 2004-2014 L2J DataPack
 * 
 * This file is part of L2J DataPack.
 * 
 * L2J DataPack is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J DataPack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package handlers.effecthandlers;

import com.l2jserver.Config;
import com.l2jserver.gameserver.model.L2Object;
import com.l2jserver.gameserver.model.StatsSet;
import com.l2jserver.gameserver.model.actor.instance.L2MonsterInstance;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.model.conditions.Condition;
import com.l2jserver.gameserver.model.effects.AbstractEffect;
import com.l2jserver.gameserver.model.holders.ItemHolder;
import com.l2jserver.gameserver.model.items.instance.L2ItemInstance;
import com.l2jserver.gameserver.model.skills.BuffInfo;
import com.l2jserver.gameserver.network.SystemMessageId;
import com.l2jserver.gameserver.network.serverpackets.InventoryUpdate;
import com.l2jserver.gameserver.network.serverpackets.ItemList;
import com.l2jserver.gameserver.network.serverpackets.SystemMessage;
import com.l2jserver.util.Rnd;

/**
 * Harvesting effect implementation.
 * @author l3x, Zoey76
 */
public final class Harvesting extends AbstractEffect
{
	public Harvesting(Condition attachCond, Condition applyCond, StatsSet set, StatsSet params)
	{
		super(attachCond, applyCond, set, params);
	}
	
	@Override
	public boolean isInstant()
	{
		return true;
	}
	
	@Override
	public void onStart(BuffInfo info)
	{
		if ((info.getEffector() == null) || (info.getEffected() == null) || !info.getEffector().isPlayer() || !info.getEffected().isNpc() || !info.getEffected().isDead())
		{
			return;
		}
		
		final L2PcInstance player = info.getEffector().getActingPlayer();
		final L2Object[] targets = info.getSkill().getTargetList(player, false, info.getEffected());
		if ((targets == null) || (targets.length == 0))
		{
			return;
		}
		
		boolean send = false;
		InventoryUpdate iu = null;
		for (L2Object target : targets)
		{
			if ((target == null) || !target.isMonster())
			{
				continue;
			}
			
			final L2MonsterInstance monster = (L2MonsterInstance) target;
			
			if (player.getObjectId() != monster.getSeederId())
			{
				player.sendPacket(SystemMessageId.YOU_ARE_NOT_AUTHORIZED_TO_HARVEST);
				continue;
			}
			if (!monster.isSeeded())
			{
				player.sendPacket(SystemMessageId.THE_HARVEST_FAILED_BECAUSE_THE_SEED_WAS_NOT_SOWN);
				continue;
			}
			if (!calcSuccess(player, monster))
			{
				player.sendPacket(SystemMessageId.THE_HARVEST_HAS_FAILED);
				continue;
			}
			
			final ItemHolder reward = monster.takeHarvest();
			if (reward != null)
			{
				if (player.isInParty())
				{
					player.getParty().distributeItem(player, reward, false, monster);
				}
				else
				{
					L2ItemInstance crop = player.getInventory().addItem("Harvesting", reward.getId(), reward.getCount(), player, monster);
					send = true;
					if (!Config.FORCE_INVENTORY_UPDATE)
					{
						if (iu == null)
							iu = new InventoryUpdate();
						iu.addItem(crop);
					}
					player.sendPacket(SystemMessage.getSystemMessage(SystemMessageId.YOU_PICKED_UP_S1_S2)
						.addItemName(reward.getId())
						.addItemNumber(reward.getCount()));
					if (player.isInParty())
						player.getParty().broadcastToPartyMembers(player, SystemMessage.getSystemMessage(SystemMessageId.C1_HARVESTED_S3_S2S)
							.addPcName(player)
							.addItemName(reward.getId())
							.addItemNumber(reward.getCount()));
				}
			}
		}
		if (send)
		{
			if (!Config.FORCE_INVENTORY_UPDATE)
			{
				if (iu != null)
					player.sendPacket(iu);
			}
			else
			{
				player.sendPacket(new ItemList(player, false));
			}
		}
	}
	
	private static boolean calcSuccess(L2PcInstance activeChar, L2MonsterInstance target)
	{
		int basicSuccess = 100;
		final int levelPlayer = activeChar.getLevel();
		final int levelTarget = target.getLevel();
		
		int diff = (levelPlayer - levelTarget);
		if (diff < 0)
		{
			diff = -diff;
		}
		
		// apply penalty, target <=> player levels
		// 5% penalty for each level
		if (diff > 5)
		{
			basicSuccess -= (diff - 5) * 5;
		}
		
		// success rate can't be less than 1%
		if (basicSuccess < 1)
		{
			basicSuccess = 1;
		}
		return Rnd.nextInt(99) < basicSuccess;
	}
}
